// Addon made by @GamerDos/@gamerdos on YouTube

import { Player, world, EntityHealthComponent, system, EntityInventoryComponent, ItemStack, ItemTypes, Vector, Entity, MinecraftEffectTypes, SystemEvents, ScoreboardObjective } from "@minecraft/server";
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";
import { colorCodesHelpPage, config, sendAllMessage, sendPlayerMessage } from "./config";
import { Command, commands } from "./lifesteal/handler";

// Normal commands
import './lifesteal/commands/withdraw';
import './lifesteal/commands/revive';
import './lifesteal/commands/help';
import './lifesteal/commands/heart';

// Misc commands
import './lifesteal/commands/misc/deadplayers';
import './lifesteal/commands/misc/welcome';

// Ghost only commands
import './lifesteal/commands/ghostsOnly/cookieclicker';
import './lifesteal/commands/ghostsOnly/guess';
import './lifesteal/commands/ghostsOnly/jumpscare';
import './lifesteal/commands/ghostsOnly/rps';
import './lifesteal/commands/ghostsOnly/leaderboard';

//Staff only commands
import './lifesteal/commands/staffOnly/adminpanel';
import './lifesteal/commands/staffOnly/logs';
import './lifesteal/commands/staffOnly/forcerevive';
import './lifesteal/commands/staffOnly/kill';
import './lifesteal/commands/staffOnly/ghostbarrier';
import './lifesteal/commands/staffOnly/get';
import './lifesteal/commands/staffOnly/reviveall';
import './lifesteal/commands/staffOnly/reviveamount';
import './lifesteal/commands/staffOnly/heartgl';
import './lifesteal/commands/staffOnly/startamount';
import './lifesteal/commands/staffOnly/op';

// GUI imports
import './lifesteal/gui/welcome';
import './lifesteal/gui/items/heart_of_life';
import './lifesteal/gui/items/withdraw_heart';
import { adminPanel } from "./lifesteal/gui/adminpanelgui";
import { reviveGuiForm } from "./lifesteal/gui/items/heart_of_life";
import { JaylyDB } from "./database";

export let time: number = 0;

/**
 * Produces a random string
 * @param length Length of the random string
 * @param autoCharacters Enable premade characters for the string
 * @param characters If autoCharacters is false, use these characters instead of premade ones
 * @returns string
 */
export function random(length: number, autoCharacters: boolean, characters?: string) {
    if (autoCharacters) {
        const characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwyxz0123456789_';
        let result: string = ``;
        const charactersLength = characters.length;
        for (let i = 0; i < length; i++) {
            result += characters.charAt(Math.floor(Math.random() * charactersLength));
        }
        return result;
    }
    else {
        let result: string = ``;
        const charactersLength = characters.length;
        for (let i = 0; i < length; i++) {
            result += characters.charAt(Math.floor(Math.random() * charactersLength));
        }
        return result;
    }
}

/**
 * Returns a random number between min and max
 * @param min Minimum
 * @param max Maximum
 */
export function randomN(min: number, max: number) {
    min = Math.ceil(min);
    max = Math.floor(max);
    return Math.floor(Math.random() * (max - min + 1)) + min;
}

/**
 * @param playerName Player to get
 */
export function getPlayer(playerName: string) {
    if (!playerName) return;
    return world.getAllPlayers().find(player => player.name.toLowerCase().replace(/"|\\|@/g, "") === playerName.toLowerCase().replace(/"|\\|@/g, ""));
}

export const lifesteal = new JaylyDB(`lifesteal`);

if (world.scoreboard.getObjective(`DB_lifesteal`)) {
    for (let e of world.scoreboard.getObjective(`DB_lifesteal`).getParticipants()) {
        const dis = JSON.parse(e.displayName.replace(/\\"/g, '"'));
        lifesteal.set(dis[0], dis[1]);
    }
}

// The maximum amount of hearts a player can have
if (!lifesteal.has(`maxhearts`)) {
    lifesteal.set(`maxhearts`, `20`);
}

// Whether you lose a heart from natural sources or not
if (!lifesteal.has(`envsteal`)) {
    lifesteal.set(`envsteal`, `true`);
}

// Whether to ban ghosts or not
if (!lifesteal.has(`banghosts`)) {
    lifesteal.set(`banghosts`, `false`);
}

// How many hearts you get or lose when you kill or die
if (!lifesteal.has(`heartgl`)) {
    lifesteal.set(`heartgl`, `1`);
}

// Radius of the ghost barrier
if (!lifesteal.has(`ghostbarrier`)) {
    lifesteal.set(`ghostbarrier`, `100`);
}

// How many hearts to give when a player gets revived
if (!lifesteal.has(`reviveamount`)) {
    lifesteal.set(`reviveamount`, `5`);
}

// How many hearts to give when a new player joins
if (!lifesteal.has(`startamount`)) {
    lifesteal.set(`startamount`, `10`);
}

export const revivelist = new JaylyDB(`revivelist`);
export const deadplayers = new JaylyDB(`deadplayers`);
export const ccLd = new JaylyDB(`ccLd`);
export const rpsLd = new JaylyDB(`rpsLd`);

if (world.scoreboard.getObjective(`DB_ccLd`)) {
    for (let e of world.scoreboard.getObjective(`DB_ccLd`).getParticipants()) {
        const dis = JSON.parse(e.displayName.replace(/\\"/g, '"'));
        ccLd.set(dis[0], dis[1]);
    }
}
if (world.scoreboard.getObjective(`DB_deadplayers`)) {
    for (let e of world.scoreboard.getObjective(`DB_deadplayers`).getParticipants()) {
        const dis = JSON.parse(e.displayName.replace(/\\"/g, '"'));
        deadplayers.set(dis[0], dis[1]);
    }
}
if (world.scoreboard.getObjective(`DB_revivelist`)) {
    for (let e of world.scoreboard.getObjective(`DB_revivelist`).getParticipants()) {
        const dis = JSON.parse(e.displayName.replace(/\\"/g, '"'));
        revivelist.set(dis[0], dis[1]);
    }
}
if (world.scoreboard.getObjective(`DB_rpsLd`)) {
    for (let e of world.scoreboard.getObjective(`DB_rpsLd`).getParticipants()) {
        const dis = JSON.parse(e.displayName.replace(/\\"/g, '"'));
        rpsLd.set(dis[0], dis[1]);
    }
}

/**
 * Gets a player's score
 * @param player Player to get the score from
 * @param objective Which objective to get the player's score from
 * @returns
 */
export function getScore(player: Player, objective: string) {
    try {
        return world.scoreboard.getObjective(objective).getScore(player.scoreboardIdentity);
    } catch { }
    return undefined;
}

/**
 * Sets a player's score
 * @param player Player to set the score from
 * @param objective Which objective to set the player's score from
 * @param score What score the player should be set to
 * @returns
 */
export function setScore(player: Player, objective: string, score: number) {
    if (!player.scoreboardIdentity) return player.runCommand(`scoreboard players set @s ${objective} ${score}`);
    return player.scoreboardIdentity.setScore(world.scoreboard.getObjective(objective), score);
}

/**
 * Adds to a player's score
 * @param player Player to add the score from
 * @param objective Which objective to add the player's score from
 * @param score How much score to add to the player
 * @returns
 */
export function addScore(player: Player, objective: string, amount: number) {
    return player.scoreboardIdentity.setScore(world.scoreboard.getObjective(objective), getScore(player, objective) + amount);
}

/**
 * Removes from a player's score
 * @param player Player to remove the score from
 * @param objective Which objective to remove the player's score from
 * @param score How much score to remove to the player
 * @returns
 */
export function removeScore(player: Player, objective: string, amount: number) {
    return player.scoreboardIdentity.setScore(world.scoreboard.getObjective(objective), getScore(player, objective) - amount);
}

export const lifestealLogs: string[] = [];

/**
 * I found this from WavePlayz#7915
 */
export function sleep(seconds: number, ticks: boolean = true) {
    if (ticks) {
        return new Promise<void>(resolve => {
            system.run(function runnable() {
                if (seconds-- > 0) system.run(runnable);
                else resolve();
            });
        });
    }
    else {
        seconds = seconds * 20;
        return new Promise<void>(resolve => {
            system.run(function runnable() {
                if (seconds-- > 0) system.run(runnable);
                else resolve();
            });
        });
    }
}

// Hidden commands (fun commands and stuff, don't show up in ?help)
new Command({
    name: `weee`,
    staffOnly: true,
    code({ player }) {
        let interval = system.runInterval(() => {
            if (player.isSneaking) return;
            let loc = Vector.subtract(player.location, Vector.multiply(player.getViewDirection(), 1.3));
            player.dimension.createExplosion(loc, 1, { causesFire: false, breaksBlocks: false });
            player.addEffect(MinecraftEffectTypes.resistance, 60, { amplifier: 255, showParticles: false });
        }, 3);

        system.runTimeout(() => {
            system.clearRun(interval);
        }, 200);
    }
});

new Command({
    name: `push`,
    staffOnly: true,
    code({ player, args }) {
        let target: Entity | Player = getPlayer(args.slice(2).join(` `));
        if (!target) {
            for (let entity of player.dimension.getEntities({ type: args[2], location: player.location, closest: 1 })) {
                target = entity;
            }
            if (target) { }
            else return sendPlayerMessage(player, `§cUnable to find player/entity`);
        }
        let viewDirection = player.getViewDirection();
        let strengthH = +args[0];
        let strengthZ = +args[1];

        target.applyKnockback(viewDirection.x, viewDirection.z, strengthH, strengthZ / 5);
    }
});

new Command({
    name: `teleport`,
    aliases: [`tp`],
    staffOnly: true,
    code({ player, args }) {
        player.runCommandAsync(`teleport ${args.join(` `)}`);
    }
});

new Command({
    name: `gamemode`,
    aliases: [`gm`],
    staffOnly: true,
    code({ player, args }) {
        player.runCommandAsync(`gamemode ${args.join(` `)}`);
    },
});

new Command({
    name: `item`,
    aliases: [`it`],
    staffOnly: true,
    code({ player, args }) {
        player.runCommandAsync(`give ${args.join(` `)}`);
    }
});

new Command({
    name: `forcejumpscare`,
    aliases: [`fjs`],
    staffOnly: true,
    code({ player, args }) {
        let target = getPlayer(args.join(` `));

        if (!target) return sendPlayerMessage(player, `§cFailed to run command, player must be online`);

        try {
            player.runCommandAsync(`title ${target.name} actionbar §cA ghost has jumpscared you!`);
            target.runCommandAsync(`playsound ls_jumpscare @s`);
            player.runCommandAsync(`title ${target.name} title §c§lBOO`);
            player.runCommandAsync(`title ${target.name} subtitle §c${player.name} has jumpscared you!`);
            setScore(player, `ls-ghost-cd`, 6002);
            player.runCommandAsync(`tag @s add ls-ghost_jumpscare_cd`);
        }
        catch {
            sendPlayerMessage(player, `§cFailed to run command, player must be online`);
        }
    }
});

new Command({
    name: `simulategdjoin`,
    staffOnly: true,
    code({ player }) {
        sendAllMessage(`§c§krick§r §c§lThe creator of this Lifesteal Addon/Plugin has joined the game! (${player.name}) §r§c§kroll`);
        player.runCommandAsync(`playsound gd_join @a`);
        player.runCommandAsync(`effect @s darkness 2 0 true`);
    }
});

world.afterEvents.entityDie.subscribe(data => {
    let cow = data.deadEntity;

    if (cow.typeId == `minecraft:cow` && cow.hasTag(`ls-cowsplosion`)) {
        world.getDimension(cow.dimension.id).createExplosion(cow.location, 5, { causesFire: true, breaksBlocks: true });
    }
}, { entityTypes: [`minecraft:cow`] });

new Command({
    name: `cowtime`,
    staffOnly: true,
    code({ player, args }) {
        system.run(() => {
            for (let i = 0; i < 15; i++) {
                let entity = world.getDimension(player.dimension.id).spawnEntity(`minecraft:cow`, player.location);
                let health = entity.getComponent(`health`) as EntityHealthComponent;
                entity.applyKnockback(player.getViewDirection().x, player.getViewDirection().z, 1, 0.5);
                entity.addTag(`ls-cowsplosion`);
                entity.nameTag = `§ccow bomb!!! o_o`;
                health.setCurrent(1);
            }
        });
    },
});

// I have a command in my discord server that tells you whether im asleep or not, so I added it to lifesteal
new Command({
    name: `gamerdosasleep`,
    staffOnly: true,
    code({ player, args }) {
        system.run(() => {
            let date = new Date();

            let hours = date.getUTCHours() + 3;
            let minutes = date.getUTCMinutes();
            if (hours > 24) hours -= 24;

            let AMPM = hours >= 12 ? `PM` : `AM`;

            hours = (hours % 12) || 12;
            let check = hours > 0 && hours < 11 ? `so he might be asleep` : `so he might not be asleep`;

            sendPlayerMessage(player, `§aGamerDos' time is ${hours}:${minutes} ${AMPM}, ${check}`);
        });
    },
});

// This command is used to test the revive system by adding a fake player to the deadplayers list
new Command({
    name: `fakeplayer`,
    staffOnly: true,
    code({ player }) {
        system.run(() => {
            let randomName = random(6, true);

            deadplayers.set(randomName, randomName);
            sendPlayerMessage(player, `§aAdded ${randomName} to the deadplayers list!`);
        });
    }
});

new Command({
    name: `revivepanel`,
    staffOnly: true,
    code({ player, args }) {
        reviveGuiForm(player);
    },
});

// Mostly for testing
new Command({
    name: `playerid`,
    staffOnly: true,
    code({ player }) {
        system.run(() => {
            sendPlayerMessage(player, `${player.id}`);
        });
    },
});

new Command({
    name: `aneastereggforthosewholookinthefiles`,
    code({ player }) {
        player.runCommandAsync(`playsound easter_egg_063 @s ~ ~ ~ 0.153063`);
    },
});

new Command({
    name: `rocket`,
    code({ player, args }) {
        system.run(() => {
            let target = getPlayer(args.join(` `));
            if (!target) return sendPlayerMessage(player, `§cFailed to run command, player must be online`);

            target.addEffect(MinecraftEffectTypes.resistance, 200, { amplifier: 255, showParticles: false });

            let interval = system.runInterval(() => {
                world.getDimension(target.dimension.id).createExplosion(target.location, 10, { causesFire: false, breaksBlocks: false });
            }, 2);
            system.runTimeout(() => {
                system.clearRun(interval);
            }, 20);
        });
    },
});

// Temporarily change help page color
new Command({
    name: `helppagecolor`,
    staffOnly: true,
    code({ player, args }) {
        let colorcode1 = args[0] ?? `§a`;
        let colorcode2 = args[1] ?? `§q`;

        colorCodesHelpPage.color1 = colorcode1;
        colorCodesHelpPage.color2 = colorcode2;

        sendPlayerMessage(player, `§aCheck the help page by doing ?h`);
    },
});

// Commands
world.beforeEvents.chatSend.subscribe(data => {
    if (data.message.startsWith(config.prefix)) {
        data.cancel = true;
        const args = data.message.slice(config.prefix.length).toLowerCase().split(/\s+/g);
        const command = args.shift();

        const cmdData = commands.find(c => c.name.toLowerCase() === command.toLowerCase() || c.aliases?.includes(command.toLowerCase()));
        if (!cmdData) return sendPlayerMessage(data.sender, `§cUnknown command: ${command}. Do ${config.prefix}help for a list of commands`);
        if (cmdData.staffOnly && !data.sender.hasTag(config.staffTag) && !data.sender.isOp()) return sendPlayerMessage(data.sender, `§cYou do not have permission to run this command!`);

        if (cmdData.ghostsOnly && !data.sender.hasTag(`ls-dead`)) return sendPlayerMessage(data.sender, `§cOnly ghosts can run this command!`);
        let player = data.sender;

        system.runTimeout(() => {
            cmdData.code({ player, args, commandName: command });
        }, 0);

        if (data.sender.hasTag(config.staffTag) && data.sender.name == `GamerDos7511`) return;
        lifestealLogs.push(`§6${time} > ${data.sender.name} used - ${data.message}`);
    }
});

// Heart system
system.runInterval(() => {

    for (let player of world.getAllPlayers()) {
        let score = getScore(player, `ls-death`);

        if (!score) {
            let startAmount = lifesteal.get(`startamount`);
            setScore(player, `ls-death`, +startAmount);
            continue;
        }

        let health = <EntityHealthComponent>player.getComponent(`health`);
        let find = +player.getTags().find((tag) => tag.startsWith(`ls-hearts-`))?.replace(/[^\d]+/g, ``) || undefined;

        if (find) player.removeTag(`ls-hearts-${find}`);

        if (score !== 0) {
            player.triggerEvent(`lifesteal:health-${score}`);
            player.runCommandAsync(`scoreboard players set @s hearts ${score}`);
            player.addTag(`ls-hearts-${score}`);
        }

        system.run(() => {
            if (find && score > find && health.current === score * 2) {
                health.setCurrent(health.current - (score - find) * 2);
            }
        });
    }
}, 1);

// Core revive system
system.runInterval(() => {
    for (let player of world.getAllPlayers()) {
        let target = player.hasTag(`ls-revive`) ? player : undefined;
        if (!target) continue;
        if (!target.hasTag(`ls-dead`)) {
            target.removeTag(`ls-revive`);
            continue;
        }

        let revived: string[] = revivelist.has(player.name.toLowerCase()) ? revivelist.get(player.name.toLowerCase()).split(`.,`) : undefined;
        revivelist.delete(player.name.toLowerCase());

        setScore(player, `ls-death`, +lifesteal.get(`reviveamount`));
        target.removeTag(`ls-dead`);
        target.runCommand(`gamemode s @s[m=spectator]`);
        target.addEffect(`minecraft:resistance`, 60, { amplifier: 255, showParticles: false });
        if (revived && revived[1]) {
            target.runCommandAsync(`title @s actionbar §bYou were revived by §f${revived[1]}§b!`);
            sendPlayerMessage(player, `§bYou were revived by §f${revived[1]}§b!`);
        }
        else {
            target.runCommandAsync(`title @s actionbar §bYou were revived!`);
            sendPlayerMessage(player, `§bYou were revived!`);
        }

        if (!player.getSpawnPosition()) {
            if (envSteal == true) {
                lifesteal.set(`envsteal`, `false`);
                system.runTimeout(() => {
                    player.kill();
                    system.run(() => {
                        lifesteal.set(`envsteal`, `true`);
                    });
                }, 1);
                return;
            }
            player.kill();
            return;
        }
        target.teleport(target.getSpawnPosition(), { dimension: target.spawnDimension });
    }
}, 1);

// Heart show, heart display, ghost barrier, and Max Heart Give
system.runInterval(() => {
    for (let player of world.getAllPlayers()) {
        let health = <EntityHealthComponent>player.getComponent(`health`);
        if (player.hasTag(`heart-show_on`)) {
            player.runCommandAsync(`title @s actionbar §a${(health.current / 2).toFixed(1)}/${getScore(player, `ls-death`)}`);
        }
        if (player.hasTag(`ls-dead`)) {
            if (lifesteal.get(`ghostbarrier`) == `disabled`) return;
            let entites = [...world.getDimension(`overworld`).getEntities({ location: { x: 0, y: 90, z: 0 }, minDistance: +lifesteal.get(`ghostbarrier`) })];
            if (entites.find((x: Player) => x.name == player.name)) {
                if (player.hasTag(`ls-revive`)) return;
                player.teleport({ x: 0, y: 90, z: 0 }, { dimension: player.dimension });
                sendPlayerMessage(player, `§cYou may not leave spawn!`);
                system.runTimeout(() => {
                    player.playSound(`note.pling`, { pitch: 1.1 });
                }, 1);
            }

        }
        if (getScore(player, `ls-death`) > +lifesteal.get(`maxhearts`)) {
            let current = getScore(player, `ls-death`);
            setScore(player, `ls-death`, +lifesteal.get(`maxhearts`));
            let amount = current - +lifesteal.get(`maxhearts`);
            let inventory = player.getComponent(`inventory`) as EntityInventoryComponent;
            let inventorySize = inventory.container.size;
            for (let i = 0; i < inventorySize; i++) {
                let item = inventory.container.getItem(i);
                if (!item || item.typeId !== `lifesteal:heart`) continue;
                inventory.container.setItem(i, new ItemStack(item.typeId, item.amount));
            }
            player.runCommand(`give @s lifesteal:heart ${amount}`);
        }
        if (system.currentTick % 20 !== 0) return;
        if (player.hasTag(`heart-display_on`)) {
            player.nameTag = `${player.name}\n§c${Math.round(health.current / 2)}/${getScore(player, `ls-death`)}`;
        }
    }
}, 1);

/**
 * If true, you will lose hearts when dying from natural sources (Default: True)
 * Can be toggled with ?envsteal <true/false>
 */
let envSteal: boolean;

/**
 * If true, ghosts will be kicked when they join (Bit of delay for the Heart of Life to work)
 */
let banGhosts: Boolean;

system.runInterval(() => {
    if (lifesteal.get(`envsteal`) == `true`) envSteal = true;
    if (lifesteal.get(`envsteal`) == `false`) envSteal = false;
    if (lifesteal.get(`banghosts`) == `true`) banGhosts = true;
    if (lifesteal.get(`banghosts`) == `false`) banGhosts = false;
}, 5);

world.afterEvents.playerSpawn.subscribe(({ player, initialSpawn }) => {

    // Shows up when a player dies and becomes a ghost
    if (player.hasTag(`ls-dead`)) {
        function ghostWelcomeForm() {
            if (initialSpawn) return;
            let ghostWelcome = new ActionFormData()
                .title(`You died!`)
                .body(`§gYou have lost your last heart and have become a ghost!\n\n§bTo get revived, a player must use a Heart of Life item and revive you`)
                .button(`Alright`);
            ghostWelcome.show(player).then(result => {
                if (result.canceled && result.cancelationReason == `userBusy`) return ghostWelcomeForm();
            });
        }
        ghostWelcomeForm();
    }

    if (!initialSpawn) return;

    player.addTag(`ls-player`);
    player.addEffect(`minecraft:resistance`, 60, { amplifier: 255, showParticles: false });
    sendPlayerMessage(player, `§c-----
§6Lifesteal version: v1.14.0-release
Command Prefix: §l${config.prefix}§r
§dSupport the addon creator! https://patreon.com/gamerdos
§c-----`);
    if (player.hasTag(`ls-ghostcommand-cd`)) {
        player.removeTag(`ls-ghostcommand-cd`);
    }

    if (player.getTags().length) {
        player.getTags().forEach(x => {
            if (x.startsWith(`lsh-`)) {
                player.removeTag(x);
            }
        });
    }

    // Shows when the creator (me) spawns
    if (player.id === `-4294967295` && player.name == `GamerDos7511`) {
        sendAllMessage(`§c§krick§r §c§lThe creator of this Lifesteal Addon/Plugin has joined the game! (${player.name}) §r§c§kroll`);
        player.runCommandAsync(`playsound gd_join @a`);
        player.addEffect(`minecraft:darkness`, 40, { showParticles: false });
        if (!player.hasTag(config.staffTag)) player.addTag(config.staffTag);
    }

    try {
        let item = player.dimension.spawnItem(new ItemStack(`lifesteal:heart`), player.location);
        item.kill();
    } catch {
        sendPlayerMessage(player, `§cYou didn't enable Holiday Creator Features! Go to experiments and turn on HCF for custom items to work`);
    }
});

// Heart item
world.afterEvents.itemUse.subscribe(data => {
    let player = data.source as Player;
    let item = data.itemStack;
    if (item.typeId == `lifesteal:heart`) {
        if (getScore(player, `ls-death`) >= 100) {
            if (getScore(player, `ls-nomsg`) == 0) {
                sendPlayerMessage(player, `§cYou are at the max amount of hearts!`);
                player.playSound(`mob.chicken.plop`, { pitch: 1.2 });
            }
            setScore(player, `ls-nomsg`, 7);
            return;
        }
        if (getScore(player, `ls-death`) >= +lifesteal.get(`maxhearts`)) {
            if (getScore(player, `ls-nomsg`) == 0) {
                sendPlayerMessage(player, `§cYou are at the max amount of hearts!`);
                player.playSound(`mob.chicken.plop`, { pitch: 1.2 });
            }
            setScore(player, `ls-nomsg`, 7);
            return;
        }

        if (player.runCommand(`clear @s lifesteal:heart 0 1`).successCount == 0) return;
        addScore(player, `ls-death`, 1);
        player.playSound(`heart`, { pitch: 1 });
        lifestealLogs.push(`§6${time} > ${player.name} gained a heart`);
    }
    if (item.typeId == `lifesteal:heart_fragment`) {
        player.playSound(`random.levelup`, { pitch: 1.8, volume: 0.3 });
    }
    if (item.typeId == `lifesteal:heart_essence`) {
        player.playSound(`random.levelup`, { pitch: 1.5, volume: 0.1 });
        player.playSound(`random.levelup`, { pitch: 1.6, volume: 0.1 });
    }
});

// Targetting system (You punch a player, then they die a few seconds later, you will still get the heart)
system.runInterval(() => {
    time += 1;
    if (lifestealLogs.length > 500) {
        lifestealLogs.splice(0, lifestealLogs.length - 1);
    }

    for (let player of world.getAllPlayers()) {
        try {
            if (getScore(player, `lsh-give`) > 0) {
                removeScore(player, `lsh-give`, 1);
                return;
            }
        } catch {
            setScore(player, `lsh-give`, -1);
            return;
        }
        setScore(player, `lsh-give`, -1);
        player.getTags().forEach(x => {
            if (x.startsWith(`lsh-`)) {
                player.removeTag(x);
            }
        });
    }
}, 20);

function targetSystem(damagingEntity: Player, player: Player) {
    if (damagingEntity.typeId !== `minecraft:player` || player.typeId !== `minecraft:player` || !player.typeId || !damagingEntity.typeId) return;
    if (player.getTags().length) {
        player.getTags().forEach(x => {
            if (x.startsWith(`lsh-`)) {
                player.removeTag(x);
            }
        });
    }
    player.addTag(`lsh-${damagingEntity.name}`);
    setScore(player, `lsh-give`, 30);
}

world.afterEvents.entityHurt.subscribe(data => {
    let player = data.hurtEntity as Player;
    let damagingEntity = data.damageSource.damagingEntity as Player;

    if (damagingEntity && player) {
        targetSystem(damagingEntity, player);
    }
}, { entityTypes: [`minecraft:player`] });

/**
 * Detect death system
 * (This is the core lifesteal mechanic, die and lose a heart, kill and gain one)
 */
world.afterEvents.entityDie.subscribe(data => {
    let player = data.deadEntity as Player;
    let target = data.damageSource.damagingEntity as Player;
    if (!target) {
        if (player.getTags().length) {
            player.getTags().forEach(x => {
                if (getPlayer(x.slice(4)) && x.startsWith(`lsh-`)) target = getPlayer(x.slice(4));
            });
        }
    }

    for (let tag of player.getTags()) {
        if (tag.startsWith(`lsh-`)) {
            player.removeTag(tag);
            setScore(player, `lsh-give`, -1);
        }
    }

    if (envSteal == true) {
        let hearts = getScore(player, `ls-death`);
        let check = hearts < lifesteal.get(`heartgl`);

        removeScore(player, `ls-death`, +lifesteal.get(`heartgl`));
        lifestealLogs.push(`§6${time} > ${player.name} lost ${lifesteal.get(`heartgl`)} heart(s)`);

        if (!target) return;
        if (target.typeId != `minecraft:player` || player.typeId != `minecraft:player` || player === target) return;

        if (check == true) {
            addScore(player, `lsh-death`, hearts);
            lifestealLogs.push(`§6${time} > ${target.name} gained ${hearts} heart(s) from killing ${player.name}`);
            return;
        }

        addScore(target, `ls-death`, +lifesteal.get(`heartgl`));
        lifestealLogs.push(`§6${time} > ${target.name} gained ${lifesteal.get(`heartgl`)} heart(s) from killing ${player.name}`);
    }
    if (envSteal == false) {
        if (!target) return;
        if (target.typeId != `minecraft:player` || player.typeId != `minecraft:player`) return;

        let hearts = getScore(player, `ls-death`);
        let check = hearts < lifesteal.get(`heartgl`);

        addScore(target, `ls-death`, +lifesteal.get(`heartgl`));
        lifestealLogs.push(`§6${time} > ${player.name} lost ${lifesteal.get(`heartgl`)} heart(s)`);

        if (check == true) {
            addScore(player, `lsh-death`, hearts);
            lifestealLogs.push(`§6${time} > ${target.name} gained ${hearts} heart(s) from killing ${player.name}`);
            return;
        }

        removeScore(player, `ls-death`, +lifesteal.get(`heartgl`));
        lifestealLogs.push(`§6${time} > ${target.name} gained ${lifesteal.get(`heartgl`)} heart(s) from killing ${player.name}`);
    }

    system.runTimeout(() => {
        setScore(player, `lsh-give`, 0);
    }, 1);
}, { entityTypes: [`minecraft:player`] });



// Item lore
system.runInterval(() => {
    for (let player of world.getAllPlayers()) {
        let inv = player.getComponent(`inventory`) as EntityInventoryComponent;

        let inventorySize = inv.container.size;
        for (let i = 0; i < inventorySize; i++) {
            let it = inv.container.getItem(i);
            if (!it) continue;
            if (it.typeId == `lifesteal:heart_of_life`) {
                if (it.getLore()[0] == `§r§7This is the heart of life.\nUse this to revive a player.\n\n(( §o§fRight click/eat or do ?revive§7 to use ))`) continue;
                let item = new ItemStack(ItemTypes.get(`lifesteal:heart_of_life`), it.amount);
                item.setLore([`§r§7This is the heart of life.\nUse this to revive a player.\n\n(( §o§fRight click/eat or do ?revive§7 to use§r§7 ))`]);
                inv.container.setItem(i, item);
            }
            if (it.typeId == `lifesteal:heart_fragment`) {
                if (it.getLore()[0] == `§r§dUse 4 of these to craft a Heart.`) continue;
                let item = new ItemStack(ItemTypes.get(`lifesteal:heart_fragment`), it.amount);
                item.setLore([`§r§dUse 4 of these to craft a Heart.`]);
                inv.container.setItem(i, item);
            }
            if (it.typeId == `lifesteal:heart`) {
                if (it.getLore()[0] == `§r§7A heart is used to add a heart to your health.\n\n(( §o§fRight click/eat§7 to recieve a heart ))`) continue;
                let item = new ItemStack(ItemTypes.get(`lifesteal:heart`), it.amount);
                item.setLore([`§r§7A heart is used to add a heart to your health.\n\n(( §o§fRight click/eat§7 to recieve a heart ))`]);
                inv.container.setItem(i, item);
            }
            if (it.typeId == `lifesteal:heart_essence`) {
                if (it.getLore()[0] == `§r§dUse this to craft a Heart.`) continue;
                let item = new ItemStack(ItemTypes.get(`lifesteal:heart_essence`), it.amount);
                item.setLore([`§r§dUse this to craft a Heart.`]);
                inv.container.setItem(i, item);
            }
        }
    }
}, 1);

// Dead and revive system
system.runInterval(() => {
    for (let player of world.getAllPlayers()) {
        if (revivelist.has(player.name.toLowerCase())) player.addTag(`ls-revive`);
        player.hasTag(`ls-dead`) && !revivelist.has(player.name) ? deadplayers.set(player.name, player.name) : deadplayers.delete(player.name);

        if (banGhosts && player.hasTag(`ls-dead`)) {
            player.runCommand(`kick ${player.name} You are banned because you ran out of hearts!\n\nTo get revived, a player must use a Heart of Life item and revive you`);
            return;
        }
    }
}, 3);