import { world, Player } from "@minecraft/server";
import { ActionFormData, ModalFormData } from "@minecraft/server-ui";
import { sendPlayerMessage, sendAllMessage, config } from "../../../config";
import { deadplayers, revivelist } from "../../../lifesteal";
import { Command } from "../../handler";

export function reviveGuiForm(player: Player) {
    let options: string[] = Array.from(deadplayers.keys());

    if (!options.length) {
        const errorGui = new ModalFormData()
            .title(`Revive GUI`)
            .textField(`Search for a player`, `e.g GamerDos7511`)
            .dropdown(`Choose a player to revive`, [`No dead players!`]);
        errorGui.show(player).then(result => {
            if (result.canceled && result.cancelationReason == `userBusy`) return reviveGuiForm(player);
            if (result.canceled && result.cancelationReason == `userClosed` || !(result.formValues[0] as any).length) return;
            sendPlayerMessage(player, `§cThere are no dead players!`);
        });
        return;
    }

    const reviveGUI = new ModalFormData()
        .title(`Revive GUI`)
        .textField(`Search for a player`, `e.g GamerDos7511`)
        .dropdown(`Choose a player to revive`, options);
    reviveGUI.show(player).then(result => {
        if (result.canceled && result.cancelationReason == `userBusy`) return reviveGuiForm(player);
        else if (result.canceled) return;
        if (result.formValues[0]) {
            let search: string[] = [];
            for (let x of options) {
                if (x.includes(<any>result.formValues[0])) search.push(x);
            }
            if (!search.length) {
                const reviveGuiSearchUnsuccessful = new ModalFormData()
                    .title(`Revive GUI - Search Error`)
                    .dropdown(`Choose a player to revive`, [`No players found!`]);
                reviveGuiSearchUnsuccessful.show(player).then(() => {
                    reviveGuiForm(player);
                });
                return;
            }
            options = search;
            reviveGuiForm(player);
            return;
        }

        let formValues = result.formValues;
        let target = options[<any>formValues[1]];

        player.runCommandAsync(`clear @s lifesteal:heart_of_life 0 1`);
        revivelist.set(`${target.toLowerCase()}`, `${target.toLowerCase()}.,${player.name}`);
        deadplayers.delete(target);
        player.triggerEvent(`lifesteal:heart_of_life_use`);
        player.runCommandAsync(`playsound heart_of_life_sound @a`);
        sendAllMessage(`§b${player.name} has revived ${target}`);
    });
}

world.afterEvents.itemUse.subscribe(data => {
    if (data.itemStack.typeId == `lifesteal:heart_of_life`) {
        let player = data.source as Player;

        reviveGuiForm(player);
    }
});